import pygame
from pygame.locals import *
from pyogre import ogre
from pygameogre import PyGameOGREApp
from behaviors import genOrbitAndRepose
from util import *

from ParallaxBackground import *
from flyers import *
from ship import Ship

class CloakApp(PyGameOGREApp):
    def init(self):
        self.last_time = pygame.time.get_ticks()
        self.back = ParallaxBackground(self.sceneManager)
        
    def createEntities(self):
        scm = self.sceneManager
#        scm.showBoundingBoxes=True
        scm.ambientLight = (0.6,0.6,0.6)
        scm.shadowTechnique = ogre.SHADOWTYPE_STENCIL_ADDITIVE

        self.ship = Ship(scm,(-10,15,0),(0,0),(1.2,1.2,1.2))

        self.enFact = EnemyFactory(scm,'ufo.mesh',(.1,.1,.1),
                        mat="Examples/EnvMappedRustySteel",
                        mover=genOrbitAndRepose(self.ship,50,70))
        self.enemies=Group([self.enFact.create((30,y,0)) for y in range(-90,90,15)])

        plane = ogre.Plane((0, 0, 1), -1000)

        mm = ogre.MeshManager.getSingleton()
        mm.createPlane('sky', ogre.ResourceManager.DEFAULT_RESOURCE_GROUP_NAME,
                       plane, 1500, 1500, 20, 20, True, 1, 5, 5, (0, -1, 0))

        ent = scm.createEntity("skyground", "sky")
        scm.rootSceneNode.createChildSceneNode().attachObject(ent)
        
        ent.setMaterialName("Examples/CloudySky")
        ent.castShadows = False

        # point light
        light = scm.createLight("Light1")
        light.type = ogre.Light.LT_DIRECTIONAL
        light.position = (30, 30, 300)
        light.diffuseColour = (.5, .5, .5)
        light.specularColour = (.8, .5, .8)
        light.direction = (0, .1, -1)

    def _createViewport(self):
        "Creates the user's viewport and camera"
        self.camera = self.sceneManager.createCamera("camera")
        self.camera.position = (0, 0, 200)
        self.camera.lookAt((0, -10, 0))
        self.camera.nearClipDistance = 5     
        self.viewport = self.renderWindow.addViewport(self.camera)
        self.viewport.backgroundColour = (0, 0, 0)

    def processEvents(self):
        for event in pygame.event.get():
            if event.type is QUIT:
                return False
            elif event.type is KEYDOWN and event.key is K_ESCAPE:
                return False
            if event.type is KEYDOWN or event.type is KEYUP:
                return self.doKey(event)
        return True

    def doKey(self, event):
        key=event.key
        ship=self.ship
        if key in [K_UP,K_DOWN,K_LEFT,K_RIGHT,K_LSHIFT]:
            return ship.steer(event)
        if event.type is KEYUP:
            if key == K_a:
                ship.toggleCloak()
            if key == K_p:
                self.renderWindow.writeContentsToFile('screenshot.tga')
            if key == K_LSHIFT:
                self.ship.fireLaser() ## Do Fire Weapon
        return True

    def update(self):
        time=pygame.time.get_ticks()
        dt=(time - self.last_time)/1000.0
        self.ship.update(dt,self)
#        if not self.ship.exploded:
#            hits=self.enemies.collide(self.ship,dt)
#            if len(hits)>0:
#                self.ship.explode()
#                for e in hits:
#                    self.enemies.remove(e)
#                    e.explode()
        for b in self.ship.l.beams:
            hits=self.enemies.collide(b,dt)
            if len(hits)>0:
                self.ship.l.beams.remove(b)
                del b
            for e in hits:
                self.enemies.remove(e)
                e.explode()  # cleans up references in scm?
        self.back.update(dt)
        self.enemies.update(dt)
        self.last_time=time

if __name__ == '__main__':
    import profile
    ca = CloakApp()
    ca.run()
#    profile.run('ca.run()')
