from util import *
from pygame.time import get_ticks
from pygame.locals import *
from pygame import mixer
import behaviors

from pyogre import ogre

class Flyer:
    def __init__(self,scm,pos=(0,0,0),vel=(0,0),scale=(1,1,1)):
        self.scm=scm
        node=scm.rootSceneNode.createChildSceneNode()
        node.position = pos
        node.scale=scale
        self.node=node
        (self.dx,self.dy)=vel
        self.sounds={}

    def update(self,dt):
        node=self.node
        oldpos=node.position
        node.position=add(scale(dt,(self.dx,self.dy,0)),oldpos)

    def collide(self, flyer, dt):
        bb1=self.node.getAttachedObject(0).worldBoundingBox
        bb2=flyer.node.getAttachedObject(0).worldBoundingBox
        vol=bb1.intersection(bb2).volume
        if vol>1000:
            print vol
            return True
        else:
            return False



#class Bullet(Flyer):
#    def _initEnt(self,id,mesh="",mat=

class Enemy(Flyer):
    def __init__(self, *args, **kw):
        if 'mover' in kw:
            self.move=kw.pop('mover')
        else:
            self.move=behaviors.genWobble(15,1)
        Flyer.__init__(self, *args, **kw)
        self.p0=tuple(self.node.position)
        self.sounds['explode']=mixer.Sound(sndPath + 'explode.wav')
        
    def _initEnt(self,id,mesh="cube.mesh",mat=None):
        scm = self.scm
        self.id="%s%d"%(mesh,id)
        self.ent = scm.createEntity(self.id, mesh)
        if mat:
            self.ent.setMaterialName(mat)
        self.node.attachObject(self.ent)
        #self.node.setDirection((0,1,0))
        self.cloak = 0

    def update(self,dt):
        self.move(self,dt)  # Mis-nomer. Runs behaviour.
        Flyer.update(self,dt)

    def toggleCloak(self):
        if self.cloak == 0:
            self.node.detachAllObjects()
            self.cloak = 1
        else:
            self.node.attachObject(self.ent)
            self.cloak = 0

    def explode(self):
        self.node.detachAllObjects()
        self.flareParticle = ogre.ParticleSystemManager.getSingleton().createSystem("Exp%s"%self.id, "Explode")
        self.node.attachObject(self.flareParticle);     
        self.sounds['explode'].play()

class EnemyFactory:
    def __init__(self,scm,mesh,scale=(1,1,1),mat=None,mover=None,
                    constructor=Enemy,type=None):
        self.count=0
        self.scm=scm
        self.mesh=mesh
        self.mat=mat
        self.mover=mover
        self.cons=constructor
        self.scale=scale
        if type:
            self.type=type
        else:
            self.type=mesh
            
    def create(self,pos,vel=(0,0),scale=(1,1,1)):
        rescale=mult(self.scale,scale)
        enemy = self.cons(self.scm,pos,vel,rescale,mover=self.mover)
        enemy._initEnt(self.count,self.mesh,self.mat)
        self.count+=1
        return enemy
    
    def reset():
        self.count=0

class Group:
    def __init__(self, items=[]):
        self.items=items
    def add(self, flyer):
        self.items.append(flyer)
    def remove(self, flyer):
        self.items.remove(flyer)
    def update(self, dt):
        for i in self.items:
            i.update(dt)
    def collide(self, flyer, dt):
        return [i for i in self.items if flyer.collide(i,dt)]
