## laser.py

from util import *
from pygame.time import get_ticks
from pygame.locals import *
from pyogre import ogre
from flyers import Flyer

class Ship(Flyer):
    def __init__(self, *args, **kw):
        Flyer.__init__(self, *args, **kw)
        self._initEnts()
        self.speed=50
        self.last_steer=0
        self.l = Laser(self)

    def _initEnts(self):
        scm=self.scm
        visiEnt = scm.createEntity("visiShip", "RZR-002.mesh")
        cloakEnt = scm.createEntity("cloakShip", "RZR-002.mesh")
        cloakEnt.setMaterialName('Examples/TransparentTest')
        cloakEnt.castShadows = False
        self.node.attachObject(visiEnt)
        self.node.setDirection((-1,0,0)) # Local orientation
        self.ents=(visiEnt,cloakEnt) # Save for toggling cloak
        self.cloak=0

    def toggleCloak(self):
        node=self.node
        self.cloak=(self.cloak+1)%2
        node.detachAllObjects()
        node.attachObject(self.ents[self.cloak])
        
    def fireLaser(self):
        self.l.fire()

    def steer(self,event):
        key=event.key
        if event.type is KEYUP:
            if key in [K_UP, K_DOWN]:
                self.dy=0
            if key in [K_LEFT, K_RIGHT]:
                self.dx=0
            return True
        if key == K_UP:
            self.dy = self.speed
        if key == K_DOWN:
            self.dy = -self.speed
        if key == K_LEFT:
            self.dx = -self.speed
        if key == K_RIGHT:
            self.dx = self.speed
        if key == K_LSHIFT:
            self.fireLaser()            
        self.last_steer=get_ticks()
        return True

class Laser(object):
    def __init__(self, ship):
        self.ship = ship
        self.scm = ship.scm
        self.lastFireLaser = 0
        self.beams = []
#        self.lnode=self.scm.rootSceneNode.createChildSceneNode()
        
    def fire(self):
        if get_ticks() - self.lastFireLaser < 100:
            return
        self.lastFireLaser = get_ticks()
#        self.sunParticle = ogre.ParticleSystemManager.getSingleton().createSystem("Exp", "Explode")
#        self.particleNode = scm.rootSceneNode.createChildSceneNode("Particle");
#        self.particleNode.attachObject(self.sunParticle);        
    # if self.laserp:
    #    ogre.ParticleSystemManager.getSingleton().destroySystem("PLaser")
        p =add((10,0,0),self.ship.node.position)
        beam = Beam(p,(400,0,0))
        self.beams.append(beam)
#        if not self.lrayq:
#            self.lrayq = self.scm.createRayQuery(self.lray)
#        else:
#            self.lrayq.ray = self.lray        
            
    def update(self, dt):
        for b in beams:
            b.update(dt)

class Beam(Flyer):
    def __init__(self, *args, **kw):
        Flyer.__init__(self, *args, **kw)
        self.lray = None
        self.lrayq = None
        self.laserp = ogre.ParticleSystemManager.getSingleton().createSystem("PLaser%d" % get_ticks(),"PLaser")
        self.node.attachObject(self.laserp)
#        self.lray = ogre.Ray()
#        self.lray.position = (xx, yy, zz)
#        self.lray.direction = (1,0,0)

    def collide(self, flyer, dt):
        (x1,y2,z2)=self.node.position
        (x2,y2,z2)=flyer.node.position
        if (x1 < x2 < x1+self.dx*dt) and (y1 < y2 < y2+self.dy*dt):
            return True
        else:
            return False
